package com.example;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.jcr.Binary;
import javax.jcr.LoginException;
import javax.jcr.PathNotFoundException;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.SimpleCredentials;
import javax.jcr.Node;
import javax.jcr.ValueFactory;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.commons.io.IOUtils;
import org.apache.jackrabbit.value.ValueFactoryImpl;

public class Warehouse {

   private static Warehouse me = new Warehouse();

   private Repository repository;

   private Warehouse() {
      try {
         InitialContext context = new InitialContext();
         Context environment = (Context) context.lookup("java:comp/env");
         repository = (Repository) environment.lookup("jcr/repository");
      } catch (NamingException e) {
         //TODO log this
      }
   }

   public static void store(String path, byte[] content)
         throws LoginException, RepositoryException {
      if (me == null) {
         throw new RepositoryException("repo failed to initialize");
      }
      me.set(path, content);
   }

   public static byte[] fetch(String path) 
      throws LoginException, RepositoryException, IOException {
      if (me == null) {
         throw new RepositoryException("repo failed to initialize");
      }
      return me.get(path);
   }




   private String[] explode(String path) {
      if ((path == null) || (path.isEmpty())) {
         throw new IllegalArgumentException("path must not be empty");
      } else if (path.endsWith("/")) {
         throw new IllegalArgumentException("path must not end with '/'");
      } else if (path.startsWith("/")) {
         path = path.substring(1);
      }
      return path.split("/");
   }

   private void set(String path, byte[] content)
         throws LoginException, RepositoryException {
      Session session = null;
      try {
         //TODO use oracle backend w/ actual credentials
         session = repository.login(new SimpleCredentials(
            "username", "password".toCharArray()));
         String[] parts = explode(path);
         Node node = session.getRootNode();
         for (int i = 0; i < parts.length; i++) {
            String part = parts[i];
            if (i == (parts.length - 1)) {
               ValueFactory vf = ValueFactoryImpl.getInstance();
               Binary b = vf.createBinary(new ByteArrayInputStream(content));
               node.setProperty(part, b);
            } else {
               // TODO disallow same-name siblings.
               try {
                  node = node.getNode(part);
               } catch (PathNotFoundException e) {
                  node = node.addNode(part);
               }
            }
         }
         session.save();
      } finally {
         if (session != null) {
            session.logout();
         }
      }
   }

   private byte[] get(String path) throws LoginException,
      RepositoryException, IOException {
      byte[] ret = null;
      Session session = null;
      try {
         session = repository.login(new SimpleCredentials(
               "username", "password".toCharArray()));
         Binary b = session.getRootNode().getProperty(path).getBinary();
         ret = IOUtils.toByteArray(b.getStream());
         b.dispose();
      } finally {
         if (session != null) {
            session.logout();
         }
      }
      return ret;
   }
}
